//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine) 
// Summer 2000
//

#include <TimeOrderedTable.h>
#include <Dal.h>
#include <string>
#include <STime.h>
#include <SasDalUtils.h>
#include <vector>
#include <cassert>
#include <cmath>

void createTimeOrderedTable(const std::string& name, const STime& tstart,
			    const long unsigned rows ,
			    const double delta = 1.0) {
  std::vector<unsigned long> size(1, 19);
  DataSet *set = dataSetServer->open(name, Dal::Create);
  Table *tab = set->addTable("tot", rows);
  Column *c = tab->addColumn("TIME", Dal::DString, "", "", size);
  ColumnData *cd = c->data();
  STime st(tstart);
  SasDalUtils::setStringColumnRow(cd, 0, st.toString(0));
  for(unsigned k = 1 ; k < rows; ++k){
    st += delta;
    SasDalUtils::setStringColumnRow(cd, k, st.toString(0));
  }
  dataSetServer->close(set);
}

int main(){

    using namespace std;

  dataSetServer->client("testtot");

  std::string sn = "testtot.ds";
  unsigned n = 1000;
  STime tstart("2000-01-01T00:00:00");

  createTimeOrderedTable(sn, tstart, n);
  DataSet *set = dataSetServer->open(sn, Dal::Read);
  Table *tab = set->table("tot");
  {
	  TimeOrderedTable tot(tab);
  
	  assert(tot.rows() == n);
	  assert(tot.validTimes() == n);

	  assert(tot.lower_bound(tstart) == n);
	  assert(tot.upper_bound(tstart) == 0);

	  assert(tot.lower_bound(tstart - 100) == n);
	  assert(tot.upper_bound(tstart - 100) == n);

	  // UL: 2001-07-18
	  // the test STime("2000-01-01T00:00:00") == STime("2001-01-01")+1.
	  // is too fragile as is replies on exact internal STime arithmetic;
	  // relaxed the following test accordingly
	  const double EPS = 1.e-10;
	  assert(tot.lower_bound(tstart + 1 + EPS) == 2);
	  assert(tot.upper_bound(tstart + 1 + EPS) == 1);

	  assert(tot.lower_bound(tstart + 1.5) == 2);
	  assert(tot.upper_bound(tstart + 1.5) == 1);

	  assert(fabs(tot.time_of_row(2) - tstart - 2.0) <= 1e-6);

	  assert(tot.row_has_time(10));
  }
  dataSetServer->close(set);
	      
}


